// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "BountyClass.h"
#include "SideBountyClass.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "GameFramework/Actor.h"
#include "BountyDirector.generated.h"

UCLASS()
class ENDLESSVENDETTA_API ABountyDirector : public AActor
{
	GENERATED_BODY()
	
	AEndlessVendettaCharacter* PlayerChar;

	// Scene Component Used to Attach Bounties to
	UArrowComponent* BountyAttachmentPoint;

	// Ordered Array of Main Bounties
	UPROPERTY(EditDefaultsOnly, Category = "Bounty")
	TArray<TSubclassOf<ABountyClass>> BountyClassArray;

	// Ordered Array of Side Bounties, in three's
	UPROPERTY(EditDefaultsOnly, Category = "Bounty")
	TArray<TSubclassOf<ASideBountyClass>> SideBountyClassArray;

	int CurrentBountyIndex = 0;

	TArray<int> CurrentSideBountyIndexes;

	UPROPERTY(VisibleAnywhere, Category = "Bounty")
	ABountyClass* ActiveBounty;

	UPROPERTY(VisibleAnywhere, Category = "Bounty")
	TArray<ASideBountyClass*> ActiveSideBounties;

	// Checks if completed the bounty, and moves onto the next if so
	UFUNCTION(BlueprintCallable, Category = "Bounty")
	void UpdateBountyProgression();

	void SpawnBountyAndItsSideBounties();
	
	// Collect reward for current Bounty and prepare for the next
	void FinishActiveBounty();
	
protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	/** Blueprint implemented function called from c++, handles aesthetic changes to bounty display
	based on data from ActiveBC and ActiveSBC */
	UFUNCTION(BlueprintImplementableEvent, Category = "bounty")
	void UpdateBountyDisplay();

	// Ran when a Side Bounty is completed and wants to update the active bounties checkpoints
	UFUNCTION()
	void SideBountyCompleted(int SideBountyIndex);

	UFUNCTION()
	void DestroyActiveSideBounties();

public:	
	// Called every frame
	virtual void Tick(float DeltaTime) override;

	// Sets default values for this actor's properties
	ABountyDirector();

	// ------ Getters ------
	UFUNCTION(BlueprintCallable, Category = "Bounty")
	FString GetBountyTitle()
	{
		return !IsValid(ActiveBounty) ? FString("N/A") : ActiveBounty->GetBountyTitle();
	}

	UFUNCTION(BlueprintCallable, Category = "Bounty")
	FString GetBountyDescription()
	{
		return !IsValid(ActiveBounty) ? FString("N/A") : ActiveBounty->GetBountyDesc();
	}

	UFUNCTION(BlueprintCallable, Category = "Bounty")
	FString GetSideBountyTitle(int SideBountyIndex)
	{
		return (SideBountyIndex > 2 || SideBountyIndex < 0 || !IsValid(ActiveSideBounties[SideBountyIndex])) ? FString("N/A") : ActiveSideBounties[SideBountyIndex]->GetBountyTitle();
	}

	UFUNCTION(BlueprintCallable, Category = "Bounty")
	FString GetSideBountyDescription(int SideBountyIndex)
	{
		return (SideBountyIndex > 2 || SideBountyIndex < 0 || !IsValid(ActiveSideBounties[SideBountyIndex])) ? FString("N/A") : ActiveSideBounties[SideBountyIndex]->GetBountyDesc();
	}

	
};
