﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "BaseWeaponClass.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "Components/CapsuleComponent.h"
#include "Kismet/GameplayStatics.h"


// Sets default values
ABaseWeaponClass::ABaseWeaponClass()
{
	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
}

// Called when the game starts or when spawned
void ABaseWeaponClass::BeginPlay()
{
	Super::BeginPlay();
	
	// Attempt to find the player character
	APlayerController* PlayerController = UGameplayStatics::GetPlayerController(this, 0);
	player = Cast<AEndlessVendettaCharacter>(PlayerController->GetCharacter());

	for (UActorComponent* actorComp : player->GetComponentsByTag(UArrowComponent::StaticClass(), FName("GunStart")))
	{
		GunStartArrow = Cast<UArrowComponent>(actorComp);
		break;
	}
	
}

// Called every frame
void ABaseWeaponClass::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

void ABaseWeaponClass::ClickDetectionTimer()
{
	if (GetWorldTimerManager().IsTimerActive(timerHandle))
	{
		return;
	}
	GetWorldTimerManager().SetTimer(timerHandle, this, &ABaseWeaponClass::TapFire, 1 / FireRate, false);
	
	
}

void ABaseWeaponClass::CancelFire()
{
	GetWorldTimerManager().ClearTimer(timerHandle);
}

void ABaseWeaponClass::TapFire()
{
	FHitResult outHit;
	FVector traceStart;
	FVector traceEnd;
	traceStart = GunStartArrow->GetComponentLocation();
	traceEnd = traceStart + (GunStartArrow->GetForwardVector() * BulletDistance);
	FCollisionQueryParams collisionParams;
	collisionParams.AddIgnoredActor(player);
	collisionParams.AddIgnoredActor(this);
	GetWorld()->LineTraceSingleByChannel(outHit, traceStart, traceEnd, ECC_Visibility, collisionParams);

	//Debug line to see where the trace hit
	DrawDebugLine(this->GetWorld(), traceStart, traceEnd, FColor::Red, true, 500.0f, 0U, 5.f);
	if (outHit.bBlockingHit)
	{
		UE_LOG(LogTemp, Display, TEXT("hit item: %s"), *outHit.GetActor()->GetName());
	}
}

void ABaseWeaponClass::HoldFire()
{
	FHitResult outHit;
	FVector traceStart;
	FVector traceEnd;
	traceStart = GunStartArrow->GetComponentLocation();
	traceEnd = traceStart + (player->GetActorForwardVector() * BulletDistance);
	FCollisionQueryParams collisionParams;
	collisionParams.AddIgnoredActor(player);
	collisionParams.AddIgnoredActor(this);
	GetWorld()->LineTraceSingleByChannel(outHit, traceStart, traceEnd, ECC_Visibility, collisionParams);

	//Debug line to see where the trace hit
	DrawDebugLine(this->GetWorld(), traceStart, traceEnd, FColor::Red, true, 500.0f, 0U, 5.f);
	if (outHit.bBlockingHit)
	{
		UE_LOG(LogTemp, Display, TEXT("hit item: %s"), *outHit.GetActor()->GetName());
	}
}
